import ROOT
import numpy as np


def sample(source, nExpected, output_name):
    '''Draws a subset of a RooDataSet.
    
    Returns:
        A RooDataSet sampled from source. The number of events is
        determined by Poisson(nExpected).

    Note:
        An event may be drawn more than once.
    '''
    nEntries = source.numEntries()
    entries_to_draw = np.random.randint(0, nEntries, size=ROOT.gRandom.Poisson(nExpected))
    drawn_dataset = ROOT.RooDataSet(output_name, '', source.get(0))
    for entry in entries_to_draw:
        drawn_dataset.add(source.get(int(entry)))
    return drawn_dataset




import pkgutil
from io import StringIO
import os
import csv
import urllib.parse


def run_list(dataType='on_resonance'):
    with StringIO(pkgutil.get_data(__package__, f'data/{dataType}.dat').decode()) as f:
        reader = csv.DictReader(f)
        for row in reader:
            yield urllib.parse.urlencode(row)


def url_list(
        dataType='on_resonance',
        eventTypes=['evtgen-uds', 'evtgen-charm', 'evtgen-charged', 'evtgen-mixed'],
        stream_cardinal=0,
        exs=None,
        data=False,
    ):
    for runs in run_list():
        ex = int(urllib.parse.parse_qs(runs)['ex'][0])
        if exs and ex not in exs: continue
        if data:
            skim = 'HadronB'  if ex < 20 else 'HadronBJ'
            yield f'http://bweb3/mdst.php?{runs}&skm={skim}&dt={dataType}&bl=caseB'
        else:
            stream_base = 10 if ex < 30 else 0
            stream = stream_base + stream_cardinal
            for eventType in eventTypes:
                yield f'http://bweb3/montecarlo.php?{runs}&ty={eventType}&dt={dataType}&bl=caseB&st={stream}'


def convert_to_stem(url):
    return '_'.join(value[0] for value in urllib.parse.parse_qs(url).values())




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="belle-analysis-tool",
    version="1.1.1",
    author="Yuan-Ru Lin",
    author_email="yuanrulin.tw@gmail.com",
    description="A package that helps you do analysis for Belle or Belle II",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Yuan-Ru-Lin/belle-analysis-tool",
    project_urls={
        "Bug Tracker": "https://github.com/Yuan-Ru-Lin/belle-analysis-tool/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Physics",
        "Intended Audience :: Science/Research",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    include_package_data=True,
    exclude_package_data={"": ["README.md"]},
    python_requires=">=3.6",
)



from contextlib import contextmanager
import subprocess
import os
import ROOT as R


@contextmanager
def canvas_upload(name='c', title='', w=1600, h=900, ncols=1, nrows=1, local_path=None, remote_path='.', keep_local_file=False):
    c = R.TCanvas(name, title, w, h)
    c.Divide(ncols, nrows)
    yield c
    fname = local_path if local_path else name + '.pdf'
    c.SaveAs(fname)
    R.gDirectory.Delete(name)
    subprocess.run(f'{os.path.expanduser("~/dropbox_uploader.sh")} upload {fname} {remote_path}'.split())
    if not keep_local_file:
        os.remove(fname)


def pretty_draw(frame):
    frame.GetXaxis().CenterTitle()
    frame.GetYaxis().CenterTitle()
    frame.GetYaxis().SetMaxDigits(3)
    frame.Draw()


import os
import abc
import sys
import configparser

import ROOT
import matplotlib as mpl
import dropbox
from dropbox.files import WriteMode
from dropbox.exceptions import ApiError, AuthError


config = configparser.ConfigParser()
config.read(os.path.expanduser('~/.config/anatool.cfg'))


class IUploader(abc.ABC):

    @abc.abstractmethod
    def upload(self, local_path, remote_path):
        pass


class DropboxUploader(IUploader):

    def __init__(self):
        self._dbx = dropbox.Dropbox(config['OAUTH_ACCESS_TOKEN']['DROPBOX'])
        try:
            self._dbx.users_get_current_account()
        except AuthError:
            sys.exit('ERROR: Invalid access token; try re-generating an access'
                     'token from the app console on the web.')
            
    def upload(self, local_path, remote_path=None):
        if remote_path is None:
            remote_path = '/' + os.path.basename(local_path)
        with open(local_path, 'rb') as f:
            try:
                self._dbx.files_upload(f.read(), remote_path, mode=WriteMode('overwrite'))
            except ApiError as err:
                print(err)
                sys.exit()


class CanvasAdapter:

    def __init__(self, canvas):
        self._canvas = canvas

    def save(self):
        if isinstance(self._canvas, ROOT.TCanvas):
            fname = self._canvas.GetName() + '.pdf'
            self._canvas.SaveAs(fname)
            return fname
        elif isinstance(self._canvas, mpl.figure.Figure):
            fname = self._canvas.get_label() + '.pdf'
            self._canvas.savefig(fname)
            return fname


class Uploader:

    def __init__(self, uploader: IUploader = DropboxUploader()):
        self._canvases = []
        self._uploader = uploader

    def register(self, canvas):
        self._canvases.append(CanvasAdapter(canvas))

    def upload(self):
        for canvas in self._canvases:
            fname = canvas.save()
            self._uploader.upload(fname)

    def __enter__(self):
        return self

    def __exit__(self, type, value, trace):
        self.upload()


